@@ -29,7 +29,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "argcontext.h"
 
 #include <pango/pango.h>
 
@@ -38,28 +37,29 @@
 #define _MAKE_FONT_NAME(family, size) family " " #size
 #define MAKE_FONT_NAME(family, size) _MAKE_FONT_NAME(family, size)
 
-char *prog_name;
+const char *prog_name;
 
 gboolean opt_display = FALSE;
 int opt_dpi = 96;
 const char *opt_font = MAKE_FONT_NAME (DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE);
 gboolean opt_header = FALSE;
-char *opt_output = NULL;
+const char *opt_output = NULL;
 int opt_margin = 10;
 int opt_markup = FALSE;
 gboolean opt_rtl = FALSE;
 int opt_rotate = 0;
 gboolean opt_auto_dir = TRUE;
-char *opt_text = NULL;
+const char *opt_text = NULL;
 gboolean opt_waterfall = FALSE;
 int opt_width = -1;
 int opt_indent = 0;
 int opt_runs = 1;
 PangoEllipsizeMode opt_ellipsize = PANGO_ELLIPSIZE_NONE;
 HintMode opt_hinting = HINT_DEFAULT;
+const char *opt_pangorc = NULL;
 
 /* Text (or markup) to render */
-char *text;
+static char *text;
 
 void
 fail (const char *format, ...)
@@ -303,28 +303,6 @@ do_output (PangoContext     *context,
     *height_out = height;
 }
 
-static void
-show_help (ArgContext *context,
-	   const char *name,
-	   const char *arg,
-	   gpointer    data) G_GNUC_NORETURN;
-
-static void
-show_help (ArgContext *context,
-	   const char *name,
-	   const char *arg,
-	   gpointer    data)
-{
-  g_print ("%s - An example viewer for Pango\n"
-	   "\n"
-	   "Syntax:\n"
-	   "    %s [options] FILE\n"
-	   "\n"
-	   "Options:\n",
-	   prog_name, prog_name);
-  arg_context_print_help (context);
-  exit (0);
-}
 
 /* This function gets called to convert a matched pattern into what
  * we'll use to actually load the font. We turn off hinting since we
@@ -343,33 +321,40 @@ fc_substitute_func (FcPattern *pattern, gpointer   data)
     }
 }
 
-void
-parse_ellipsis (ArgContext *arg_context,
-		const char *name,
+static gboolean
+parse_ellipsis (const char *name,
 		const char *arg,
-		gpointer    data)
+		gpointer    data,
+		GError **error)
 {
   static GEnumClass *class = NULL;
+  gboolean ret = TRUE;
   GEnumValue *value;
 
   if (!class)
     class = g_type_class_ref (PANGO_TYPE_ELLIPSIZE_MODE);
   
   value = g_enum_get_value_by_nick (class, arg);
-  if (!value)
-    fail ("--ellipsize option must be one of none/start/middle/end");
+  if (value)
+    opt_ellipsize = value->value;
+  else
+    {
+      fail ("--ellipsize option must be one of none/start/middle/end");
+      ret = FALSE;
+    }
 
-  opt_ellipsize = value->value;
+  return ret;
 }
 
-void
-parse_hinting (ArgContext *arg_context,
-	       const char *name,
+static gboolean
+parse_hinting (const char *name,
 	       const char *arg,
-	       gpointer    data)
+	       gpointer    data,
+	       GError    **error)
 {
   static GEnumClass *class = NULL;
-
+  gboolean ret = TRUE;
+  
   if (!class)
     class = g_type_class_ref (PANGO_TYPE_ELLIPSIZE_MODE);
 
@@ -380,74 +365,86 @@ parse_hinting (ArgContext *arg_context,
   else if (strcmp (arg, "full") == 0)
     opt_hinting = HINT_FULL;
   else
-    fail ("--hinting option must be one of none/auto/full");
+    {
+      fail ("--hinting option must be one of none/auto/full");
+      ret = FALSE;
+    }
+
+  return ret;
 }
 
 void
 parse_options (int argc, char *argv[])
 {
-  static const ArgDesc args[] = {
-    { "no-auto-dir","Don't set layout direction according to contents",
-      ARG_NOBOOL,   &opt_auto_dir, NULL },
-    { "display",    "Show output using ImageMagick",
-      ARG_BOOL,     &opt_display, NULL },
-    { "dpi",        "Set the dpi'",
-      ARG_INT,      &opt_dpi, NULL },
-    { "ellipsize",  "Ellipsization mode [=none/start/middle/end]",
-      ARG_CALLBACK, NULL, parse_ellipsis },
-    { "font",       "Set the font name",
-      ARG_STRING,   &opt_font, NULL },
-    { "header",     "Display the options in the output",
-      ARG_BOOL,     &opt_header, NULL },
-    { "help",       "Show this output",
-      ARG_CALLBACK, NULL, show_help },
-    { "hinting",    "Hinting style [=none/auto/full]",
-      ARG_CALLBACK, NULL, parse_hinting },
-    { "margin",     "Set the margin on the output in pixels",
-      ARG_INT,      &opt_margin, NULL },
-    { "markup",     "Interpret contents as Pango markup",
-      ARG_BOOL,     &opt_markup, NULL },
-    { "output",     "Name of output file",
-      ARG_STRING,   &opt_output, NULL },
-    { "rtl",        "Set base dir to RTL",
-      ARG_BOOL,     &opt_rtl, NULL },
-    { "rotate",     "Angle at which to rotate results",
-      ARG_INT,      &opt_rotate, NULL },
-    { "text",       "Text to display (instead of a file)",
-      ARG_STRING,   &opt_text, NULL },
-    { "waterfall",  "Create a waterfall display",
-      ARG_BOOL,     &opt_waterfall, NULL },
-    { "width",      "Width in points to which to wrap output",
-      ARG_INT,      &opt_width, NULL },
-    { "indent",     "Width in points to indent paragraphs",
-      ARG_INT,      &opt_indent, NULL },
-    { "runs",       "Render text this many times",
-      ARG_INT,      &opt_runs, NULL },
-    { NULL, NULL, 0, NULL, NULL }
+  GOptionEntry entries[] = 
+  {
+    {"no-auto-dir",	0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,	&opt_auto_dir,
+     "No layout direction according to contents",			NULL },
+    {"display",	0, 0, G_OPTION_ARG_NONE,				&opt_display,
+     "Show output using ImageMagick", 					NULL },
+    {"dpi",		0, 0, G_OPTION_ARG_INT,				&opt_dpi,
+     "Set the dpi", 							NULL },  
+    {"ellipsize",	0, 0, G_OPTION_ARG_CALLBACK,			&parse_ellipsis,
+     "Ellipsization mode",  "start/middle/end"				     },  
+    {"font",		0, 0,G_OPTION_ARG_STRING,			&opt_font,
+     "Set the font name", 						NULL },  
+    {"header",		0, 0, G_OPTION_ARG_NONE,			&opt_header,
+     "Display the options in the output",				NULL },
+    {"hinting",	0, 0, G_OPTION_ARG_CALLBACK,				&parse_hinting,
+     "Hinting style", "none/auto/full"					     },  
+    {"indent",		0, 0, G_OPTION_ARG_INT,				&opt_indent,
+     "Width in points to indent paragraphs", 				NULL },  
+    {"margin",		0, 0, G_OPTION_ARG_INT,				&opt_margin,
+     "Set the margin on the output in pixels", 				NULL },  
+    {"makup",		0, 0, G_OPTION_ARG_NONE,			&opt_markup,
+     "Interpret contents as Pango markup", 				NULL },
+    {"output",		0, 0,G_OPTION_ARG_STRING,			&opt_output,
+     "Name of output file", 						NULL },  
+    {"pangorc",		0, 0,G_OPTION_ARG_STRING,			&opt_pangorc,
+     "pangorc file to use (default is ./pangorc)", 			NULL },  
+    {"rtl",		0, 0, G_OPTION_ARG_NONE,			&opt_rtl,
+     "Set base dir to RTL", 						NULL },
+    {"rotate",		0, 0, G_OPTION_ARG_INT,				&opt_rotate,
+     "Angle at which to rotate results", 				NULL },  
+    {"runs",		0, 0, G_OPTION_ARG_INT,				&opt_runs,
+     "Render text this many times", 					NULL },  
+    {"text",		0, 0,G_OPTION_ARG_STRING,			&opt_text,
+     "Text to display (instead of a file)", 				NULL },  
+    {"waterfall",	0, 0, G_OPTION_ARG_NONE,			&opt_waterfall,
+     "Create a waterfall display", 					NULL },
+    {"width",		0, 0, G_OPTION_ARG_INT,				&opt_width,
+     "Width in points to which to wrap output", 			NULL },  
+    {NULL }
   };
-
-  ArgContext *arg_context;
   GError *error = NULL;
+  GError *parse_error = NULL;
+  GOptionContext *context;
   size_t len;
   char *p;
 
   prog_name = g_path_get_basename (argv[0]);
-  
-  arg_context = arg_context_new (NULL);
-  arg_context_add_table (arg_context, args);
-
-  if (!arg_context_parse (arg_context, &argc, &argv, &error))
-    fail ("%s", error->message);
-
-  arg_context_free (arg_context);
-  
-  if ((opt_text && argc != 1) ||
-      (!opt_text && argc != 2))
+  context = g_option_context_new ("- FILE");
+  g_option_context_add_main_entries (context, entries, NULL);
+  if (!g_option_context_parse (context, &argc, &argv, &parse_error))
+  {
+    if (parse_error != NULL)
     {
-      if (opt_text && argc == 2)
+      fail("Parse option error : %s\n", parse_error->message);
+    }
+    else
+    {
+      fail("Parse option error\n");
+    }
+    exit(1);
+  }
+  g_option_context_free(context);
+
+  if ((opt_text && argc != 1) || (!opt_text && argc != 2))
+    {
+      if (opt_text && argc != 1)
 	fail ("When specifying --text, no file should be given");
 	
-      g_printerr ("Usage: %s [options] FILE\n", prog_name);
+      g_printerr ("Usage: %s [OPTION...] FILE\n", prog_name);
       exit (1);
     }
   
@@ -488,8 +485,17 @@ parse_options (int argc, char *argv[])
   if (opt_markup &&
       !pango_parse_markup (text, -1, 0, NULL, NULL, NULL, &error))
     fail ("Cannot parse input as markup: %s", error->message);
+
+  /* Setup PANGO_RC_FILE
+   */
+  if (!opt_pangorc)
+    if (g_file_test ("./pangorc", G_FILE_TEST_IS_REGULAR))
+      opt_pangorc = "./pangorc";
+  if (opt_pangorc)
+    g_setenv ("PANGO_RC_FILE", opt_pangorc, TRUE);
 }
 
+
 void
 finalize (void)
 {
