@@ -20,6 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 
 #include "arabic-ot.h"
@@ -179,8 +180,7 @@ fallback_shape (PangoEngineShape *engine,
 		PangoGlyphString *glyphs)
 {
   PangoFcFont *fc_font = PANGO_FC_FONT (font);
-  glong n_chars;
-  gunichar *wcs = g_utf8_to_ucs4_fast (text, length, &n_chars);
+  glong n_chars = g_utf8_strlen (text, length);
   const char *p;
   int i;
   
@@ -246,8 +246,6 @@ fallback_shape (PangoEngineShape *engine,
       /* Swap all glyphs */
       swap_range (glyphs, 0, glyphs->num_glyphs);
     }
-
-  g_free (wcs);
 }
 
 static void 
@@ -258,16 +256,16 @@ arabic_engine_shape (PangoEngineShape *engine,
 		     PangoAnalysis    *analysis,
 		     PangoGlyphString *glyphs)
 {
-  glong n_chars;
-  gunichar *wcs;
-  int i;
-  const char *p;
-  gulong *properties = NULL;
+  PangoFcFont *fc_font;
   FT_Face face;
   PangoOTRuleset *ruleset;
   PangoOTBuffer *buffer;
-  PangoFcFont *fc_font;
+  gulong *properties = NULL;
+  glong n_chars;
+  gunichar *wcs;
+  const char *p;
   int cluster = 0;
+  int i;
 
   g_return_if_fail (font != NULL);
   g_return_if_fail (text != NULL);
@@ -275,9 +273,8 @@ arabic_engine_shape (PangoEngineShape *engine,
   g_return_if_fail (analysis != NULL);
 
   fc_font = PANGO_FC_FONT (font);
-
   face = pango_fc_font_lock_face (fc_font);
-  g_assert (face);
+  g_return_if_fail (face != NULL);
 
   ruleset = get_ruleset (face);
   if (!ruleset)
@@ -294,6 +291,8 @@ arabic_engine_shape (PangoEngineShape *engine,
   properties = g_new0 (gulong, n_chars);
       
   Arabic_Assign_Properties (wcs, properties, n_chars);
+
+  g_free (wcs);
   
   p = text;
   for (i=0; i < n_chars; i++)
@@ -316,7 +315,7 @@ arabic_engine_shape (PangoEngineShape *engine,
 	    input = buf;
 	  }
 
-      if (wc >= 0x200B && wc <= 0x200F)	/* Zero-width characters */
+      if (pango_is_zero_width (wc))	/* Zero-width characters */
 	{
 	  pango_ot_buffer_add_glyph (buffer, 0, properties[i], p - text);
 	}
@@ -356,7 +355,6 @@ arabic_engine_shape (PangoEngineShape *engine,
   pango_ot_ruleset_position (ruleset, buffer);
   pango_ot_buffer_output (buffer, glyphs);
   
-  g_free (wcs);
   g_free (properties);
   pango_ot_buffer_destroy (buffer);
 
