@@ -21,6 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 
 #include <pango/pango-ot.h>
@@ -294,14 +295,15 @@ hebrew_engine_shape (PangoEngineShape *engine,
 		     PangoAnalysis    *analysis,
 		     PangoGlyphString *glyphs)
 {
-  const gchar *p;
-  gint i;
-  glong n_chars;
-  gint unknown_property = 0;
+  PangoFcFont *fc_font;
   FT_Face face;
   PangoOTRuleset *ruleset;
   PangoOTBuffer *buffer;
-  PangoFcFont *fc_font;
+  gint unknown_property = 0;
+  glong n_chars;
+  const char *p;
+  int cluster = 0;
+  int i;
 
   g_return_if_fail (font != NULL);
   g_return_if_fail (text != NULL);
@@ -309,9 +311,8 @@ hebrew_engine_shape (PangoEngineShape *engine,
   g_return_if_fail (analysis != NULL);
 
   fc_font = PANGO_FC_FONT (font);
-
   face = pango_fc_font_lock_face (fc_font);
-  g_assert (face);
+  g_return_if_fail (face != NULL);
 
   ruleset = get_ruleset (face);
   if (!ruleset)
@@ -326,7 +327,6 @@ hebrew_engine_shape (PangoEngineShape *engine,
   n_chars = g_utf8_strlen (text, length);
       
   p = text;
-
   for (i=0; i < n_chars; i++)
     {
       gunichar wc;
@@ -334,7 +334,6 @@ hebrew_engine_shape (PangoEngineShape *engine,
       PangoGlyph index;
       char buf[6];
       const char *input;
-      int cluster = 0;
 
       wc = g_utf8_get_char (p);
       input = p;
@@ -347,19 +346,26 @@ hebrew_engine_shape (PangoEngineShape *engine,
 	    input = buf;
 	  }
 
-      index = pango_fc_font_get_glyph (fc_font, wc);
-      
-      if (!index)
+      if (pango_is_zero_width (wc))	/* Zero-width characters */
 	{
-	  pango_ot_buffer_add_glyph (buffer, pango_fc_font_get_unknown_glyph (fc_font, wc),
-				     unknown_property, p - text);
+	  pango_ot_buffer_add_glyph (buffer, 0, unknown_property, p - text);
 	}
       else
 	{
-	  cluster = p - text;
+	  index = pango_fc_font_get_glyph (fc_font, wc);
 	  
-	  pango_ot_buffer_add_glyph (buffer, index,
-				     unknown_property, cluster);
+	  if (!index)
+	    {
+	      pango_ot_buffer_add_glyph (buffer, pango_fc_font_get_unknown_glyph (fc_font, wc),
+					 unknown_property, p - text);
+	    }
+	  else
+	    {
+	      cluster = p - text;
+	      
+	      pango_ot_buffer_add_glyph (buffer, index,
+					 unknown_property, cluster);
+	    }
 	}
       
       p = g_utf8_next_char (p);
