@@ -24,6 +24,7 @@
 #include <math.h>
 #include <string.h>
 
+#include "pango-impl-utils.h"
 #include "pango-types.h"
 #include "pango-font.h"
 #include "pango-fontmap.h"
@@ -85,7 +86,7 @@ static const PangoFontDescription pfd_defaults = {
 PangoFontDescription *
 pango_font_description_new (void)
 {
-  PangoFontDescription *desc = g_new (PangoFontDescription, 1);
+  PangoFontDescription *desc = g_slice_new (PangoFontDescription);
 
   *desc = pfd_defaults;
 
@@ -596,7 +597,7 @@ pango_font_description_better_match (const PangoFontDescription *desc,
 PangoFontDescription *
 pango_font_description_copy  (const PangoFontDescription  *desc)
 {
-  PangoFontDescription *result = g_new (PangoFontDescription, 1);
+  PangoFontDescription *result = g_slice_new (PangoFontDescription);
 
   *result = *desc;
 
@@ -624,7 +625,7 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
 PangoFontDescription *
 pango_font_description_copy_static (const PangoFontDescription *desc)
 {
-  PangoFontDescription *result = g_new (PangoFontDescription, 1);
+  PangoFontDescription *result = g_slice_new (PangoFontDescription);
 
   *result = *desc;
   if (result->family_name)
@@ -723,7 +724,7 @@ pango_font_description_free  (PangoFontDescription  *desc)
       if (desc->family_name && !desc->static_family)
 	g_free (desc->family_name);
 
-      g_free (desc);
+      g_slice_free (PangoFontDescription, desc);
     }
 }
 
@@ -883,7 +884,7 @@ pango_font_description_from_string (const char *str)
 
   g_return_val_if_fail (str != NULL, NULL);
 
-  desc = g_new (PangoFontDescription, 1);
+  desc = g_slice_new (PangoFontDescription);
 
   desc->mask = PANGO_FONT_MASK_STYLE |
                PANGO_FONT_MASK_WEIGHT |
@@ -1163,7 +1164,30 @@ pango_font_get_glyph_extents  (PangoFont      *font,
 			       PangoRectangle *ink_rect,
 			       PangoRectangle *logical_rect)
 {
-  g_return_if_fail (font != NULL);
+  if (G_UNLIKELY (font == NULL))
+    {
+      
+      if (!_pango_warning_history.get_glyph_extents)
+        {
+	  _pango_warning_history.get_glyph_extents = TRUE;
+	  g_critical ("pango_font_get_glyph_extents called with font == NULL, expect ugly output");
+	}
+      if (ink_rect)
+        {
+	  ink_rect->x = 0;
+	  ink_rect->y = 0;
+	  ink_rect->height = 12 * PANGO_SCALE;
+	  ink_rect->width = 12 * PANGO_SCALE;
+        }
+      if (logical_rect)
+        {
+	  logical_rect->x = 0;
+	  logical_rect->y = 0;
+	  logical_rect->height = 12 * PANGO_SCALE;
+	  logical_rect->width = 12 * PANGO_SCALE;
+	}
+      return;
+    }
 
   PANGO_FONT_GET_CLASS (font)->get_glyph_extents (font, glyph, ink_rect, logical_rect);
 }
@@ -1180,7 +1204,7 @@ pango_font_get_glyph_extents  (PangoFont      *font,
  * be provided to indicate that the metrics should be retrieved that
  * correspond to the script(s) used by that language.
  *
- * Returns: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
+ * Return value: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
 PangoFontMetrics *
@@ -1196,7 +1220,7 @@ pango_font_get_metrics (PangoFont        *font,
  * 
  * Gets the font map for which the font was created.
  *
- * Returns: the #PangoFontMap for the font
+ * Return value: the #PangoFontMap for the font
  *
  * Since: 1.10
  **/
@@ -1236,7 +1260,7 @@ pango_font_metrics_get_type (void)
 PangoFontMetrics *
 pango_font_metrics_new (void)
 {
-  PangoFontMetrics *metrics = g_new0 (PangoFontMetrics, 1);
+  PangoFontMetrics *metrics = g_slice_new0 (PangoFontMetrics);
   metrics->ref_count = 1;
 
   return metrics;
@@ -1277,7 +1301,7 @@ pango_font_metrics_unref (PangoFontMetrics *metrics)
   metrics->ref_count--;
   
   if (metrics->ref_count == 0)
-    g_free (metrics);
+    g_slice_free (PangoFontMetrics, metrics);
 }
 
 /**
