@@ -160,7 +160,7 @@ pango_context_set_matrix (PangoContext       *context,
  * Gets the transformation matrix that will be applied when
  * rendering with this context. See pango_context_set_matrix().
  *
- * Returns: the matrix, or %NULL if no matrix has been set
+ * Return value: the matrix, or %NULL if no matrix has been set
  *  (which is the same as the identity matrix). The returned
  *  matrix is owned by Pango and must not be modified or
  *  freed.
@@ -443,7 +443,7 @@ static void
 shaper_font_cache_destroy (ShaperFontCache *cache)
 {
   g_hash_table_destroy (cache->hash);
-  g_free (cache);
+  g_slice_free (ShaperFontCache, cache);
 }
 
 static void
@@ -466,7 +466,7 @@ get_shaper_font_cache (PangoFontset *fontset)
   cache = g_object_get_qdata (G_OBJECT (fontset), cache_quark);
   if (!cache)
     {
-      cache = g_new (ShaperFontCache, 1);
+      cache = g_slice_new (ShaperFontCache);
       cache->hash = g_hash_table_new_full (g_direct_hash, NULL,
 					   NULL, (GDestroyNotify)shaper_font_element_destroy);
 
@@ -1368,7 +1368,7 @@ update_metrics_from_items (PangoFontMetrics *metrics,
  * be a composite of the metrics for the fonts loaded for the
  * individual families.
  *
- * Returns: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
+ * Return value: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
 PangoFontMetrics *
