@@ -25,12 +25,13 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "pango-font.h"
+#include "pango-impl-utils.h"
 #include "pango-utils.h"
+#include "pango-font.h"
 
 #include <glib/gstdio.h>
 
-#include <mini-fribidi/fribidi.h>
+#include "mini-fribidi/fribidi.h"
 
 #ifndef HAVE_FLOCKFILE
 #  define flockfile(f) (void)1
@@ -57,6 +58,8 @@ struct PangoAlias
 
 static GHashTable *pango_aliases_ht = NULL;
 
+PangoWarningHistory _pango_warning_history = {};
+
 /**
  * pango_trim_string:
  * @str: a string
@@ -1157,14 +1160,17 @@ pango_matrix_concat (PangoMatrix       *matrix,
  * pango_matrix_get_font_scale_factor:
  * @matrix: a #PangoMatrix, may be %NULL
  * 
- * Return value: the scale factor of @matrix on the height of the font.
+ * Returns the scale factor of a matrix on the height of the font.
  * That is, the scale factor in the direction perpendicular to the
- * vector that the X coordinate is mapped to, or 1.0 if @matrix is %NULL.
+ * vector that the X coordinate is mapped to.
+ *
+ * Return value: the scale factor of @matrix on the height of the font,
+ * or 1.0 if @matrix is %NULL.
  *
  * Since: 1.12
  **/
 double
-pango_matrix_get_font_scale_factor (PangoMatrix *matrix)
+pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
 {
 /*
  * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
@@ -1336,7 +1342,7 @@ pango_language_from_string (const char *language)
  * or the range is a prefix of the tag, and the character after the
  * tag is '-'.
  *
- * Returns: %TRUE if a match was found.
+ * Return value: %TRUE if a match was found.
  **/
 gboolean
 pango_language_matches (PangoLanguage *language,
@@ -1458,7 +1464,6 @@ pango_log2vis_get_embedding_levels (const gchar    *str,
 				    PangoDirection *pbase_dir)
 {
   FriBidiCharType fribidi_base_dir;
-  gboolean result;
   guint8 *embedding_levels_list;
 
   switch (*pbase_dir)
@@ -1580,7 +1585,7 @@ alias_free (struct PangoAlias *alias)
 
   g_free (alias->families);
   
-  g_free (alias);
+  g_slice_free (struct PangoAlias, alias);
 }
 
 static void
@@ -1662,7 +1667,7 @@ read_alias_file (const char *filename)
       
       if (!alias)
 	{
-	  alias = g_new0 (struct PangoAlias, 1);
+	  alias = g_slice_new0 (struct PangoAlias);
 	  alias->alias = alias_key.alias;
 	  
 	  g_hash_table_insert (pango_aliases_ht,
