@@ -22,8 +22,10 @@
 
 #include <config.h>
 
+#include "pango-fontmap.h"
 #include "pangoatsui-private.h"
 #include "pango-utils.h"
+#include "modules.h"
 
 #import <Cocoa/Cocoa.h>
 
@@ -171,6 +173,7 @@ static void
 pango_atsui_family_class_init (PangoFontFamilyClass *class)
 {
   GObjectClass *object_class = (GObjectClass *)class;
+  int i;
   
   pango_atsui_family_parent_class = g_type_class_peek_parent (class);
 
@@ -179,6 +182,9 @@ pango_atsui_family_class_init (PangoFontFamilyClass *class)
   class->list_faces = pango_atsui_family_list_faces;
   class->get_name = pango_atsui_family_get_name;
   class->is_monospace = pango_atsui_family_is_monospace;
+
+  for (i = 0; _pango_included_atsui_modules[i].list; i++)
+    pango_module_register (&_pango_included_atsui_modules[i]);
 }
 
 static void
@@ -434,13 +440,13 @@ font_hash_key_free (FontHashKey *key)
     PANGO_ATSUI_FONT_MAP_GET_CLASS (key->fontmap)->context_key_free (key->fontmap,
 								     key->context_key);
   
-  g_free (key);
+  g_slice_free (FontHashKey, key);
 }
 
 static FontHashKey *
 font_hash_key_copy (FontHashKey *old)
 {
-  FontHashKey *key = g_new (FontHashKey, 1);
+  FontHashKey *key = g_slice_new (FontHashKey);
   
   key->fontmap = old->fontmap;
   key->matrix = old->matrix;
