@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <math.h>
 #include <stdlib.h>
@@ -38,13 +38,24 @@
 #define PANGO_CAIRO_IS_FONT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_CAIRO_FC_FONT))
 #define PANGO_CAIRO_FC_FONT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_CAIRO_FC_FONT, PangoCairoFcFontClass))
 
-#define PANGO_CAIRO_UNKNOWN_FLAG 0x10000000
-
 typedef struct _PangoCairoFcFont      PangoCairoFcFont;
 typedef struct _PangoCairoFcFontClass PangoCairoFcFontClass;
-typedef struct _PangoCairoFcGlyphInfo PangoCairoFcGlyphInfo;
+
+typedef struct _GlyphExtentsCacheEntry    GlyphExtentsCacheEntry;
+typedef struct _GUnicharToGlyphCacheEntry GUnicharToGlyphCacheEntry;
+
+
+/* define one of these to profile one of the caches */
+#undef PROFILE_GLYPH_EXTENTS_CACHE
+#undef PROFILE_CHAR_TO_GLYPH_CACHE
 
 #undef PROFILE_GLYPH_CACHE
+#ifdef PROFILE_GLYPH_EXTENTS_CACHE
+# define PROFILE_GLYPH_CACHE GlyphExtentsCacheEntry
+#endif
+#ifdef PROFILE_CHAR_TO_GLYPH_CACHE
+# define PROFILE_GLYPH_CACHE GUnicharToGlyphCacheEntry
+#endif
 #ifdef PROFILE_GLYPH_CACHE
 static long num_cairo_fc_fonts;
 static long max_cairo_fc_fonts;
@@ -54,21 +65,35 @@ static long num_glyph_cache_hits;
 static long num_glyph_cache_misses;
 #endif
 
-#define GLYPH_CACHE_NUM_ENTRIES 256
-#define GLYPH_CACHE_MASK 0x000000ff
 
-/* An entry in the fixed-size cache for the gunichar -> glyph mapping.  The
- * cache is indexed by the lower N bits of the gunichar (see
- * GLYPH_CACHE_NUM_ENTRIES and GLYPH_CACHE_MASK above).  For scripts with few
- * code points, this should provide pretty much instant lookups.
+
+#define GLYPH_CACHE_NUM_ENTRIES 256 /* should be power of two */
+#define GLYPH_CACHE_MASK (GLYPH_CACHE_NUM_ENTRIES - 1)
+
+/* An entry in the fixed-size cache for the gunichar -> glyph mapping.
+ * The cache is indexed by the lower N bits of the gunichar (see
+ * GLYPH_CACHE_NUM_ENTRIES).  For scripts with few code points,
+ * this should provide pretty much instant lookups.
  *
  * The "ch" field is zero if that cache entry is invalid.
  */
-typedef struct
+struct _GUnicharToGlyphCacheEntry
 {
-  gunichar ch;
+  gunichar   ch;
   PangoGlyph glyph;
-} GlyphCacheEntry;
+};
+
+/* An entry in the fixed-size cache for the glyph -> ink_rect mapping.
+ * The cache is indexed by the lower N bits of the glyph (see
+ * GLYPH_CACHE_NUM_ENTRIES).  For scripts with few glyphs,
+ * this should provide pretty much instant lookups.
+ */
+struct _GlyphExtentsCacheEntry
+{
+  PangoGlyph     glyph;
+  int            width;
+  PangoRectangle ink_rect;
+};
 
 struct _PangoCairoFcFont
 {
@@ -80,9 +105,9 @@ struct _PangoCairoFcFont
   cairo_matrix_t ctm;
   cairo_font_options_t *options;
 
-  GHashTable *glyph_info;
-
-  GlyphCacheEntry *char_to_glyph_cache;
+  PangoRectangle font_extents;
+  GlyphExtentsCacheEntry    *glyph_extents_cache;
+  GUnicharToGlyphCacheEntry *char_to_glyph_cache;
 };
 
 struct _PangoCairoFcFontClass
@@ -90,12 +115,6 @@ struct _PangoCairoFcFontClass
   PangoFcFontClass  parent_class;
 };
 
-struct _PangoCairoFcGlyphInfo
-{
-  PangoRectangle logical_rect;
-  PangoRectangle ink_rect;
-};
-
 GType pango_cairo_fc_font_get_type (void);
 
 static cairo_font_face_t *pango_cairo_fc_font_get_font_face (PangoCairoFont *font);
@@ -118,7 +137,7 @@ pango_cairo_fc_font_get_font_face (PangoCairoFont *font)
        * we can't proceed at that point
        */
       if (!cffont->font_face)
-	g_error ("Unable create Cairo font name");
+	g_error ("Unable to create FT2 cairo font face.\nThis means out of memory or a cairo/fontconfig/FreeType bug");
     }
   
   return cffont->font_face;
@@ -144,7 +163,7 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
        * we can't proceed at that point
        */
       if (!cffont->scaled_font)
-	g_error ("Unable create Cairo font");
+	g_error ("Unable to create FT2 cairo scaled font.\nThis means out of memory or a cairo/fontconfig/FreeType bug");
     }
   
   return cffont->scaled_font;
@@ -170,6 +189,8 @@ static void
 cairo_font_iface_init (PangoCairoFontIface *iface)
 {
   iface->install = pango_cairo_fc_font_install;
+  iface->get_font_face = pango_cairo_fc_font_get_font_face;
+  iface->get_scaled_font = pango_cairo_fc_font_get_scaled_font;
 }
 
 G_DEFINE_TYPE_WITH_CODE (PangoCairoFcFont, pango_cairo_fc_font, PANGO_TYPE_FC_FONT,
@@ -187,12 +208,20 @@ pango_cairo_fc_font_finalize (GObject *object)
   if (cffont->options)
     cairo_font_options_destroy (cffont->options);
 
-  g_hash_table_destroy (cffont->glyph_info);
+
+  if (cffont->glyph_extents_cache)
+    {
+      g_free (cffont->glyph_extents_cache);
+#ifdef PROFILE_GLYPH_EXTENTS_CACHE
+      num_glyph_caches--;
+      g_assert (num_glyph_caches >= 0);
+#endif
+    }
 
   if (cffont->char_to_glyph_cache)
     {
       g_free (cffont->char_to_glyph_cache);
-#ifdef PROFILE_GLYPH_CACHE
+#ifdef PROFILE_CHAR_TO_GLYPH_CACHE
       num_glyph_caches--;
       g_assert (num_glyph_caches >= 0);
 #endif
@@ -206,85 +235,6 @@ pango_cairo_fc_font_finalize (GObject *object)
 #endif
 }
 
-static void
-compute_glyph_extents (PangoFont        *font,
-		       PangoGlyph        glyph,
-		       PangoRectangle   *ink_rect,
-		       PangoRectangle   *logical_rect)
-{
-  PangoCairoFont *cfont = (PangoCairoFont *)font;
-  cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
-
-  /* It may well be worth caching the font_extents here, since getting them
-   * is pretty expensive.
-   */
-  cairo_font_extents_t font_extents;
-  cairo_text_extents_t extents;
-  cairo_glyph_t cairo_glyph;
-  
-  cairo_scaled_font_extents (scaled_font, &font_extents);
-  
-  logical_rect->x = 0;
-  logical_rect->y = - font_extents.ascent * PANGO_SCALE;
-  logical_rect->width = 0;
-  logical_rect->height = (font_extents.ascent + font_extents.descent) * PANGO_SCALE;
-
-  if (glyph)
-    {
-      cairo_glyph.index = glyph;
-      cairo_glyph.x = 0;
-      cairo_glyph.y = 0;
-      
-      cairo_scaled_font_glyph_extents (scaled_font,
-				       &cairo_glyph, 1, &extents);
-      
-      ink_rect->x = extents.x_bearing * PANGO_SCALE;
-      ink_rect->y = extents.y_bearing * PANGO_SCALE;
-      ink_rect->width = extents.width * PANGO_SCALE;
-      ink_rect->height = extents.height * PANGO_SCALE;
-      
-      logical_rect->width = extents.x_advance * PANGO_SCALE;
-    }
-}
-
-static PangoCairoFcGlyphInfo *
-pango_cairo_fc_font_get_glyph_info (PangoFont   *font,
-				    PangoGlyph   glyph)
-{
-  PangoCairoFcFont *cffont = (PangoCairoFcFont *)font;
-  PangoCairoFcGlyphInfo *info;
-
-  info = g_hash_table_lookup (cffont->glyph_info, GUINT_TO_POINTER (glyph));
-  if (!info)
-    {
-      info = g_new0 (PangoCairoFcGlyphInfo, 1);
-      
-      compute_glyph_extents (font, glyph,
-			     &info->ink_rect,
-			     &info->logical_rect);
-      
-      g_hash_table_insert (cffont->glyph_info, GUINT_TO_POINTER (glyph), info);
-    }
-
-  return info;
-}
-     
-static void
-pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
-				       PangoGlyph      glyph,
-				       PangoRectangle *ink_rect,
-				       PangoRectangle *logical_rect)
-{
-  PangoCairoFcGlyphInfo *info;
-
-  info = pango_cairo_fc_font_get_glyph_info (font, glyph);
-  
-  if (ink_rect)
-    *ink_rect = info->ink_rect;
-  if (logical_rect)
-    *logical_rect = info->logical_rect;
-}
-
 /* This function is cut-and-pasted from pangocairo-fcfont.c - it might be
  * better to add a virtual fcfont->create_context (font).
  */
@@ -317,10 +267,9 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
       if (!fcfont->fontmap)
 	return pango_font_metrics_new ();
 
-      info = g_new0 (PangoFcMetricsInfo, 1);
+      info = g_slice_new0 (PangoFcMetricsInfo);
       
-      fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, 
-						 info);
+      fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, info);
 	
       info->sample_str = sample_str;
 
@@ -360,17 +309,14 @@ pango_cairo_fc_font_get_glyph (PangoFcFont *font,
 {
   PangoCairoFcFont *cffont = (PangoCairoFcFont *)font;
   guint idx;
-  GlyphCacheEntry *entry;
+  GUnicharToGlyphCacheEntry *entry;
 
-  /* We create the cache dynamically because although we'll have a lot of
-   * PangoCairoFcFont structures (one per font in the fontset, as many as needed
-   * to cover the whole Unicode space), we'll only end up using a few of them
-   * for the user's actual text.
-   */
   if (cffont->char_to_glyph_cache == NULL)
     {
-      cffont->char_to_glyph_cache = g_new0 (GlyphCacheEntry, GLYPH_CACHE_NUM_ENTRIES);
-#ifdef PROFILE_GLYPH_CACHE
+      cffont->char_to_glyph_cache = g_new0 (GUnicharToGlyphCacheEntry, GLYPH_CACHE_NUM_ENTRIES);
+      /* Make sure all cache entries are invalid initially */
+      cffont->char_to_glyph_cache[0].ch = 1; /* char 1 cannot happen in bucket 0 */
+#ifdef PROFILE_CHAR_TO_GLYPH_CACHE
       num_glyph_caches++;
       if (num_glyph_caches > max_glyph_caches)
 	max_glyph_caches = num_glyph_caches;
@@ -380,22 +326,22 @@ pango_cairo_fc_font_get_glyph (PangoFcFont *font,
   idx = wc & GLYPH_CACHE_MASK;
   entry = cffont->char_to_glyph_cache + idx;
 
-  if (entry->ch != wc || wc == 0)
+  if (entry->ch != wc)
     {
-#ifdef PROFILE_GLYPH_CACHE
+#ifdef PROFILE_CHAR_TO_GLYPH_CACHE
       num_glyph_cache_misses++;
-/*       printf ("cache MISS: cffont %p, gunichar %x = '%c'\n", cffont, wc, (wc < 128) ? wc : 0); */
+      /*g_message ("cache MISS: cffont %p, gunichar %x = '%c'", cffont, wc, (wc < 128) ? wc : 0);*/
 #endif
       entry->ch = wc;
       entry->glyph = ((PangoFcFontClass *) pango_cairo_fc_font_parent_class)->get_glyph (font, wc);
     }
+#ifdef PROFILE_CHAR_TO_GLYPH_CACHE
   else
     {
-#ifdef PROFILE_GLYPH_CACHE
       num_glyph_cache_hits++;
-/*       printf ("cache HIT: cffont %p, gunichar %x = '%c'\n", cffont, wc, (wc < 128) ? wc : 0); */
-#endif
+      /*g_message ("cache HIT: cffont %p, gunichar %x = '%c'", cffont, wc, (wc < 128) ? wc : 0);*/
     }
+#endif
 
   return entry->glyph;
 }
@@ -404,7 +350,124 @@ static PangoGlyph
 pango_cairo_fc_font_real_get_unknown_glyph (PangoFcFont *font,
 					    gunichar     wc)
 {
-  return 0;
+  return wc | PANGO_CAIRO_UNKNOWN_FLAG;
+}
+
+static void
+pango_cairo_fc_font_glyph_extents_cache_init (PangoCairoFcFont *cffont)
+{
+  PangoCairoFont *cfont = (PangoCairoFont *)cffont;
+  cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
+  cairo_font_extents_t font_extents;
+  cairo_scaled_font_extents (scaled_font, &font_extents);
+
+  cffont->font_extents.x = 0;
+  cffont->font_extents.y = - font_extents.ascent * PANGO_SCALE;
+  cffont->font_extents.height = (font_extents.ascent + font_extents.descent) * PANGO_SCALE;
+  cffont->font_extents.width = 0;
+
+  cffont->glyph_extents_cache = g_new0 (GlyphExtentsCacheEntry, GLYPH_CACHE_NUM_ENTRIES);
+  /* Make sure all cache entries are invalid initially */
+  cffont->glyph_extents_cache[0].glyph = 1; /* glyph 1 cannot happen in bucket 0 */
+  
+#ifdef PROFILE_GLYPH_EXTENTS_CACHE
+  num_glyph_caches++;
+  if (num_glyph_caches > max_glyph_caches)
+    max_glyph_caches = num_glyph_caches;
+#endif
+}
+
+/* Fills in the glyph extents cache entry
+ */
+static void
+compute_glyph_extents (PangoCairoFcFont       *cffont,
+		       PangoGlyph              glyph,
+		       GlyphExtentsCacheEntry *entry)
+{
+  cairo_text_extents_t extents;
+  cairo_glyph_t cairo_glyph;
+
+  cairo_glyph.index = glyph;
+  cairo_glyph.x = 0;
+  cairo_glyph.y = 0;
+  
+  cairo_scaled_font_glyph_extents (cffont->scaled_font,
+				   &cairo_glyph, 1, &extents);
+  
+  entry->glyph = glyph;
+  entry->width = extents.x_advance * PANGO_SCALE;
+  entry->ink_rect.x = extents.x_bearing * PANGO_SCALE;
+  entry->ink_rect.y = extents.y_bearing * PANGO_SCALE;
+  entry->ink_rect.width = extents.width * PANGO_SCALE;
+  entry->ink_rect.height = extents.height * PANGO_SCALE;
+}
+     
+static GlyphExtentsCacheEntry *
+pango_cairo_fc_font_get_glyph_extents_cache_entry (PangoCairoFcFont       *cffont,
+						   PangoGlyph              glyph)
+{
+  GlyphExtentsCacheEntry *entry;
+  guint idx;
+
+  idx = glyph & GLYPH_CACHE_MASK;
+  entry = cffont->glyph_extents_cache + idx;
+
+  if (entry->glyph != glyph)
+    {
+#ifdef PROFILE_GLYPH_EXTENTS_CACHE
+      num_glyph_cache_misses++;
+      /*g_message ("cache MISS: cffont %p, glyph %x", cffont, glyph);*/
+#endif
+      compute_glyph_extents (cffont, glyph, entry);
+    }
+#ifdef PROFILE_GLYPH_EXTENTS_CACHE
+  else
+    {
+      num_glyph_cache_hits++;
+      /*g_message ("cache HIT: cffont %p, glyph %x = '%c'", cffont, glyph);*/
+    }
+#endif
+
+  return entry;
+}
+
+static void
+pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
+				       PangoGlyph      glyph,
+				       PangoRectangle *ink_rect,
+				       PangoRectangle *logical_rect)
+{
+  PangoCairoFcFont *cffont = (PangoCairoFcFont *)font;
+  GlyphExtentsCacheEntry *entry;
+
+  /* We need to initialize the cache here, since we use cffont->font_extents
+   */
+  if (cffont->glyph_extents_cache == NULL)
+    pango_cairo_fc_font_glyph_extents_cache_init (cffont);
+
+  if (glyph & PANGO_CAIRO_UNKNOWN_FLAG) 
+    {
+      _pango_cairo_get_glyph_extents_missing((PangoCairoFont *)font, glyph, ink_rect, logical_rect);
+      return;
+    }
+  else if (!glyph)
+    {
+      if (ink_rect)
+	*ink_rect = cffont->font_extents;
+      if (logical_rect)
+	*logical_rect = cffont->font_extents;
+      return;
+    }
+
+  entry = pango_cairo_fc_font_get_glyph_extents_cache_entry (cffont, glyph);
+
+  if (ink_rect)
+    *ink_rect = entry->ink_rect;
+  if (logical_rect)
+    {
+      *logical_rect = cffont->font_extents;
+      logical_rect->width = entry->width;
+    }
 }
 
 static void
@@ -427,11 +490,12 @@ pango_cairo_fc_font_shutdown (PangoFcFont *fcfont)
 static void
 profile_glyph_cache_exit_cb (void)
 {
-  printf ("Maximum number of PangoCairoFcFont objects: %ld\n", max_cairo_fc_fonts);
-  printf ("Maximum number of glyph caches created: %ld (%ld KB)\n", max_glyph_caches, (max_glyph_caches * sizeof (GlyphCacheEntry) * GLYPH_CACHE_NUM_ENTRIES) / 1024);
-  printf ("Glyph cache hits: %ld\n", num_glyph_cache_hits);
-  printf ("Glyph cache misses: %ld\n", num_glyph_cache_misses);
-  printf ("Glyph cache miss rate: %f%%\n", 100.0 * (double) num_glyph_cache_misses / (num_glyph_cache_misses + num_glyph_cache_hits));
+  g_message ("Profiled %s cache", G_STRINGIFY (PROFILE_GLYPH_CACHE));
+  g_message ("Maximum number of PangoCairoFcFont objects: %ld", max_cairo_fc_fonts);
+  g_message ("Maximum number of glyph caches created: %ld (%ld KB)", max_glyph_caches, (max_glyph_caches * sizeof (PROFILE_GLYPH_CACHE) * GLYPH_CACHE_NUM_ENTRIES) / 1024);
+  g_message ("Glyph cache hits: %ld", num_glyph_cache_hits);
+  g_message ("Glyph cache misses: %ld", num_glyph_cache_misses);
+  g_message ("Glyph cache miss rate: %f%%", 100.0 * (double) num_glyph_cache_misses / (num_glyph_cache_misses + num_glyph_cache_hits));
 }
 #endif
 
@@ -461,11 +525,6 @@ pango_cairo_fc_font_class_init (PangoCairoFcFontClass *class)
 static void
 pango_cairo_fc_font_init (PangoCairoFcFont *cffont)
 {
-  cffont->glyph_info = g_hash_table_new_full (g_direct_hash,
-					      NULL,
-					      NULL,
-					      (GDestroyNotify)g_free);
-
 #ifdef PROFILE_GLYPH_CACHE
   num_cairo_fc_fonts++;
   if (num_cairo_fc_fonts > max_cairo_fc_fonts)
@@ -481,7 +540,8 @@ static double
 get_font_size (PangoCairoFcFontMap        *cffontmap,
 	       PangoContext               *context,
 	       const PangoFontDescription *desc,
-	       FcPattern                  *pattern)
+	       FcPattern                  *pattern,
+	       const PangoMatrix          *matrix)
 {
   double size;
 
@@ -499,7 +559,7 @@ get_font_size (PangoCairoFcFontMap        *cffontmap,
   */
 
   if (FcPatternGetDouble (pattern, FC_PIXEL_SIZE, 0, &size) == FcResultMatch)
-    return size * PANGO_SCALE;
+    return size * PANGO_SCALE / pango_matrix_get_font_scale_factor (matrix);
 
   /* Just in case FC_PIXEL_SIZE got unset between pango_fc_make_pattern()
    * and here.
@@ -548,12 +608,13 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
   else
     cairo_matrix_init_identity (&cffont->font_matrix);
 
-  size = get_font_size (cffontmap, context, desc, pattern);
+  pango_ctm = pango_context_get_matrix (context);
+
+  size = get_font_size (cffontmap, context, desc, pattern, pango_ctm);
 
   cairo_matrix_scale (&cffont->font_matrix,
 		      size / PANGO_SCALE, size / PANGO_SCALE);
 
-  pango_ctm = pango_context_get_matrix (context);
   if (pango_ctm)
     cairo_matrix_init (&cffont->ctm,
 		       pango_ctm->xx,
@@ -564,6 +625,7 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
   else
     cairo_matrix_init_identity (&cffont->ctm);
 
+
   cffont->options = cairo_font_options_copy (_pango_cairo_context_get_merged_font_options (context));
   
   return PANGO_FC_FONT (cffont);
