@@ -19,10 +19,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "pangocairo.h"
 #include "pangocairo-private.h"
+#include "pango-utils.h"
 
 #if defined (HAVE_CAIRO_ATSUI)
 #  include "pangocairo-atsui.h"
@@ -54,7 +55,7 @@ pango_cairo_font_map_get_type (void)
       };
 
       cairo_font_map_type =
-	g_type_register_static (G_TYPE_INTERFACE, "PangoCairoFontMap",
+	g_type_register_static (G_TYPE_INTERFACE, I_("PangoCairoFontMap"),
 				&cairo_font_map_info, 0);
 
       g_type_interface_add_prerequisite (cairo_font_map_type, PANGO_TYPE_FONT_MAP);
@@ -218,7 +219,7 @@ free_context_info (PangoCairoContextInfo *info)
   if (info->merged_options)
     cairo_font_options_destroy (info->merged_options);
   
-  g_free (info);
+  g_slice_free (PangoCairoContextInfo, info);
 }
 
 static PangoCairoContextInfo *
@@ -235,7 +236,7 @@ get_context_info (PangoContext *context,
 
   if (!info && create)
     {
-      info = g_new (PangoCairoContextInfo, 1);
+      info = g_slice_new (PangoCairoContextInfo);
       info->dpi = -1.0;
       info->set_options = NULL;
       info->surface_options = NULL;
@@ -385,7 +386,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 }
   
 /**
- * pango_cairo_get_font_options:
+ * pango_cairo_context_get_font_options:
  * @context: a #PangoContext, from pango_cairo_font_map_create_context()
  * 
  * Retrieves any font rendering options previously set with
