@@ -19,6 +19,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include "pangofc-font.h"
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
@@ -110,7 +112,7 @@ static void
 free_metrics_info (PangoFcMetricsInfo *info)
 {
   pango_font_metrics_unref (info->metrics);
-  g_free (info);
+  g_slice_free (PangoFcMetricsInfo, info);
 }
 
 static void
@@ -465,7 +467,7 @@ pango_fc_font_get_metrics (PangoFont     *font,
       if (!fcfont->fontmap)
 	return pango_font_metrics_new ();
 
-      info = g_new0 (PangoFcMetricsInfo, 1);
+      info = g_slice_new0 (PangoFcMetricsInfo);
       
       fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, 
 						 info);
@@ -530,7 +532,7 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
  * This face will be kept around until you call
  * pango_fc_font_unlock_face().
  *
- * Returns: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType FT_Face associated with @font.
  *
  * Since: 1.4
  **/
@@ -704,7 +706,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
  *
  * This will return any custom decoder set on this font.
  *
- * Returns: The custom decoder
+ * Return value: The custom decoder
  *
  * Since: 1.6
  **/
