@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -194,7 +195,7 @@ pango_x_font_map_get_type (void)
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_MAP,
-                                            "PangoXFontMap",
+                                            I_("PangoXFontMap"),
                                             &object_info, 0);
     }
   
@@ -267,7 +268,7 @@ static GList *fontmaps = NULL;
  * not be freed. If the font map for a display is no longer needed, it can 
  * be released with pango_x_shutdown_display().
  *
- * Returns: a #PangoXFontMap for @display.
+ * Return value: a #PangoXFontMap for @display.
  **/
 PangoFontMap *
 pango_x_font_map_for_display (Display *display)
@@ -1338,7 +1339,7 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
  *
  * Obtains the font cache associated with the given font map.
  *
- * Returns: the #PangoXFontCache of @font_map.
+ * Return value: the #PangoXFontCache of @font_map.
  **/
 PangoXFontCache *
 pango_x_font_map_get_font_cache (PangoFontMap *font_map)
@@ -1510,7 +1511,7 @@ pango_x_face_get_type (void)
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FACE,
-                                            "PangoXFace",
+                                            I_("PangoXFace"),
                                             &object_info, 0);
     }
   
@@ -1669,7 +1670,7 @@ pango_x_family_get_type (void)
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FAMILY,
-                                            "PangoXFamily",
+                                            I_("PangoXFamily"),
                                             &object_info, 0);
     }
   
