@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <stdlib.h>
 
@@ -268,6 +268,12 @@ typedef struct
   PangoRectangle logical_rect;
 } Extents;
 
+static void
+extents_free (Extents *ext)
+{
+  g_slice_free (Extents, ext);
+}
+
 static void
 get_glyph_extents_raw (PangoXftFont     *xfont,
 		       PangoGlyph        glyph,
@@ -278,14 +284,14 @@ get_glyph_extents_raw (PangoXftFont     *xfont,
 
   if (!xfont->glyph_info)
     xfont->glyph_info = g_hash_table_new_full (NULL, NULL,
-					       NULL, (GDestroyNotify)g_free);
+					       NULL, (GDestroyNotify)extents_free);
 
   extents = g_hash_table_lookup (xfont->glyph_info,
 				 GUINT_TO_POINTER (glyph));
 
   if (!extents)
     {
-      extents = g_new (Extents, 1);
+      extents = g_slice_new (Extents);
      
       pango_fc_font_get_raw_extents (PANGO_FC_FONT (xfont),
 				     FT_LOAD_NO_BITMAP | FT_LOAD_NO_HINTING,
@@ -475,7 +481,7 @@ pango_xft_font_real_shutdown (PangoFcFont *fcfont)
  *
  * Returns the XftFont of a font.
  *
- * Returns: the XftFont associated to @font.
+ * Return value: the XftFont associated to @font.
  **/
 XftFont *
 pango_xft_font_get_font (PangoFont *font)
@@ -491,7 +497,7 @@ pango_xft_font_get_font (PangoFont *font)
  *
  * Returns the X display of the XftFont of a font.
  *
- * Returns: the X display of the XftFont associated to @font.
+ * Return value: the X display of the XftFont associated to @font.
  **/
 Display *
 pango_xft_font_get_display (PangoFont *font)
@@ -538,7 +544,7 @@ pango_xft_font_get_unknown_glyph (PangoFont *font,
  *
  * Use pango_fc_font_lock_face() instead.
  *
- * Returns: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType FT_Face associated with @font.
  *
  * Since: 1.2
  **/
