@@ -409,7 +409,7 @@ pango_xft_renderer_draw_glyph (PangoRenderer   *renderer,
 			       double           x,
 			       double           y)
 {
-  g_error ("pango_xft_renderer_draw_glyph(): should not be called\n");
+  g_critical ("pango_xft_renderer_draw_glyph(): should not be called");
 }
 
 static void
@@ -676,7 +676,7 @@ pango_xft_render_layout_line (XftDraw         *draw,
  * @draw:    an #XftDraw
  * @color:   the color in which to draw the glyphs
  * @font:    the font in which to draw the string
- * @matrix: a #PangoMatrix, or %NULL to use an identity transformation
+ * @matrix:  a #PangoMatrix, or %NULL to use an identity transformation
  * @glyphs:  the glyph string to draw
  * @x:       the x position of the start of the string (in Pango
  *           units in user space coordinates)
